%
%   bwc_create.m   this function is designed using Tom Michaud's inverse kinetics
%          code in a function file in order to take reactivity and height vectors of
%          a given blade and generate coefficients for a blade worth curve model.
%          The integral blade worth model that these coefficients fit to is:
%             c(1)x^4 + c(2)x^3 + c(3)x^2 + c(4)x^1 + c(5)x^0 + c(6)sin(2pi*x)
%   Note; The location data, x, is normalized for the actual curve fit to hopefully
%   improve the condition of the normal eqns (these are notoriously ill-conditioned)

      function [c,r2,z_FTR,rhoinv_FTR] = bwc_create(rhoinv_BOI,z,Bcat,H)
      [z_FTR,rhoinv_FTR] = filter_data(rhoinv_BOI,z,Bcat);      % filter out repetitive data
      [z_FTR,ind] = sort(z_FTR);  rhoinv_FTR = rhoinv_FTR(ind); % sort data from low to high z
      rhoinv_FTR = rhoinv_FTR*100;                              % put into %Dk/k units
      x = z_FTR;  y = rhoinv_FTR; N = length(x);                % x,y values for curve fit
      HH = H(Bcat);  xx = x/HH;                                 % normalized x
%
%   Linear Least Squares Fit to a Combined Polynomial and Sinusoid Model
      if N >= 7
        expr = {'x.^4','x.^3','x.^2','x','1','sin(2*pi*x)'};
        options = fitoptions('Method','LinearLeastSquares','Normalize','off');
        PSM = fittype(expr);
        [cfun,good] = fit(xx,y,PSM,options);
        c = coeffvalues(cfun);  r2 = good.rsquare;    % coeffs for normalized x
        c(1) = c(1)/HH^4;  c(2) = c(2)/HH^3;          % coeffs for regular x
        c(3) = c(3)/HH^2;  c(4) = c(4)/HH;            % (these are used for the plots)
      else
        disp(' ')
        disp('   No curve fit was performed because the BOI is not moving (i.e. select another BOI...)')
        disp(' ')
        c = [];  r2 = [];
      end
%
%   end of routine