%
%   rebank_adjust.m  
%
%   This is a modified version of Tom Michaud's Rebank_Adjust code.  I cleaned 
%   things up a bit and put in a few more comments, but I have to admit that I 
%   really don't follow all the logic here.  To help in understanding what Tom has 
%   done I have added a debug capability, where
%      debug = 0 or 1  for option to say NO or YES to a set of debug plots
%   Once everything is working okay, debug will be set to 0 so that there is no 
%   additional plots  --  but the capability will be retained so that I can always 
%   come back at a later time to look at the details again if desired...
%                                                            JRWhite June 2012
%
%   Well we are having difficulty with this routine not giving correct results for 
%   all cases.  I have decided to re-program Tom's original code, since I can't
%   figure out exactly what he is doing -- especially at the end points (this is
%   where the difficulty appears - I think).  Here goes...
%   
%   After a bit of work, I think I got this now!!!  The code is much more
%   straightforward and I got rid of the debug plots (not necessary any longer)..
%                                                            JRWhite Aug 2012
%
%
%   Last modified by JRWhite, UMass-Lowell (August 2012)

      function [rhoinv_boi] = rebank_adjust(rhoinv_bra,z,t,BOI)
%
      do_ra = 1;                 % set to do a rebank adjust
      [Nt,Nblades] = size(z);    % # of time pts, # of blades
%
%   check to see if the other blades are moving and, if not, skip the rebank calc
      num = 0;                   % counts # of times z(t) is moving (not including BOI)
      for i = 1:Nblades
        if i ~= BOI
          num = num + sum(z(:,i) ~= z(1,i));
        end
      end
      if num == 0,  
        do_ra = 0;
        disp(' ')
        disp('   No rebank adjustment was performed because only the BOI is moving...')
        disp(' ')
      end
% 
%   aslo skip if BOI is not moving
      numboi = 0;                  % counts # of times BOI is moving 
      numboi = numboi + sum(z(:,BOI) ~= z(1,BOI));
      if numboi == 0,  
        do_ra = 0;
        disp(' ')
        disp('   No rebank adjustment was performed because the BOI is not moving...')
        disp(' ')
      end
%
      if do_ra > 0
%
%   index(i) --  0/1 - BOI is/is not moving, where i is the time index
        index = zeros(Nt,1);
        index(1) = 1;
        for i = 1:Nt-3
          if z(i+1,BOI) == z(i,BOI) && z(i+2,BOI) == z(i,BOI) && z(i+3,BOI) == z(i,BOI)
            index(i+1) = 1;   % points where blade of interest (BOI) is stationary
          end
        end
%
%   index2(j) = time point where blade either starts or stops
        index2 = zeros(Nt,1);
        j = 1;
        for ii = 1:Nt-3
          if abs(index(ii)-index(ii+1)) > 0
            j = j+1;
            index2(j) = ii;  % pts where BOI has started or stopped moving
          end
        end
        index2(j+1:Nt) = [];  % remove elements not used
%
% checks if length of index2 is even and cuts off last number      
        if rem(length(index2),2) == 0 
          index2 = index2(1:(end)-1,:);  
        end
%
%   brkpt(1:2,i) = breakpoint matrix containing time pts where breaks occur
%   Note: The time point indicated here has the blade at the same location as the
%   previous point. Thus, for the start of motion, brkpt+1 is the location of first 
%   movement.  At the end, however, brkpt-1 is really the last point of movement...)
        Nbrks = floor(length(index2)/2);         
        brkpt = zeros(2,Nbrks);  
        for i = 1:Nbrks
          brkpt(1,i) = index2(2*i);     % 1st row has pts where BOI motion starts 
          brkpt(2,i) = index2(2*i+1);   % 2nd row has pts where BOI motion stops
        end
%
        clear index index2              % index and index2 no longer used
%
%   Note: Tom's logic for setting the break points seems to work -- so I left this
%   essentially intact.  My only change affected the first and last points. 
       brkpt = brkpt+1;
%   Note: Upon careful comparison, it appears that the break pt array is out of sync
%   with the rhoinv_bra array by one time point (probably because of the zero time??).
%   In any case, adding one unit to the brkpt array semms to set things straight (for
%   my new rebank adjust algorithm below...).
%
%   The idea behind the "rebank adjust" is to determine the reactivity associated
%   with the BOI from the total.  The logic here assumes that only one blade is moved
%   at a time. Thus, we find where the BOI is moving (in the above brkpt array) and
%   only accumlate reactivity for the BOI during times of movement.
%
        rhoinv_boi = zeros(Nt,1);                           % initialize rho for BOI
        for k = 1:Nbrks
          istrt1 = brkpt(1,k)+1;   iend1 = brkpt(2,k)-1;
          range1 = istrt1:iend1;                            % while BOI is moving
          drho = rhoinv_bra(range1)- rhoinv_bra(istrt1-1);
          rhoinv_boi(range1) = rhoinv_boi(istrt1-1)+drho;
          istrt2 = iend1+1;                                 % while BOI is not moving
          if k < Nbrks                                     
            iend2 = brkpt(1,k+1);
          else
            iend2 = Nt;
          end
          range2 = istrt2:iend2;    
          rhoinv_boi(range2) = rhoinv_boi(iend1);
        end
%
      else
        rhoinv_boi = rhoinv_bra;   % initialize rho for BOI
      end   % end of do_ra test
%
%   end of routine